@@ -6,12 +6,11 @@ abspath = os.path.abspath(os.path.dirname(__file__))
 if abspath not in sys.path:
     sys.path.insert(0, abspath)
 
-import requests
 from server import FHIRServer, FHIRUnauthorizedException
 from auth import FHIRAuth
 import models.patient as patient
 
-__version__ = '0.0.2'
+__version__ = '0.0.3'
 __author__ = 'SMART Platforms Team'
 __license__ = 'APACHE2'
 __copyright__ = "Copyright 2014 Boston Children's Hospital"
@@ -19,7 +18,6 @@ __copyright__ = "Copyright 2014 Boston Children's Hospital"
 scope_default = 'user/*.* patient/*.read openid profile'
 scope_nolaunch = 'launch/patient'
 
-
 class FHIRClient(object):
     """ Instances of this class handle authorizing and talking to SMART on FHIR
     servers.
@@ -28,11 +26,11 @@ class FHIRClient(object):
     
         - `app_id`: Your app/client-id, e.g. 'my_web_app'
         - `api_base`: The SMART service to connect to, e.g. 'https://fhir-api.smartplatforms.org'
-        - `auth_type`: The authorization type, supports "oauth2". Defaults to "oauth2" if omitted
+        - `auth_type`: The authorization type, supports "oauth2" and "none". Defaults to "oauth2" if omitted
         - `redirect_uri`: The callback/redirect URL for your app, e.g. 'http://localhost:8000/fhir-app/' when testing locally
     """
     
-    def __init__(self, settings=None, state=None):
+    def __init__(self, settings=None, state=None, save_func=lambda x:x):
         self.app_id = None
         self.server = None
         self.auth = None
@@ -56,10 +54,16 @@ class FHIRClient(object):
             
             auth_type = settings.get('auth_type')
             redirect = settings.get('redirect_uri')
-            self.auth = self._auth_for_type(auth_type, scope=scope, redirect_uri=redirect)
+            patient_id = settings.get('patient_id')
+            if patient_id:
+                self.auth = FHIRAuth.create(auth_type, app_id=self.app_id, patient_id=patient_id)
+            else:
+                self.auth = FHIRAuth.create(auth_type, app_id=self.app_id, scope=scope, redirect_uri=redirect)
         else:
             raise Exception("Must either supply settings or a state upon client initialization")
     
+        self._save_func = save_func
+        self.save_state()
     
     # MARK: Authorization
     
@@ -67,11 +71,6 @@ class FHIRClient(object):
     def auth_type(self):
         return self.auth.auth_type if self.auth else None
     
-    def _auth_for_type(self, auth_type, **kwargs):
-        if auth_type is None:
-            auth_type = 'oauth2'
-        return FHIRAuth.create(auth_type, app_id=self.app_id, **kwargs)
-    
     @property
     def ready(self):
         """ Returns True if the client is ready to make API calls (e.g. there
@@ -83,7 +82,9 @@ class FHIRClient(object):
     def authorize_url(self):
         """ The URL to use to receive an authorization token.
         """
-        return self.auth.authorize_url(self.server) if self.auth is not None else None
+        auth_url = self.auth.authorize_url(self.server) if self.auth is not None else None
+        self.save_state()
+        return auth_url
     
     def handle_callback(self, url):
         """ You can call this to have the client automatically handle the
@@ -93,7 +94,8 @@ class FHIRClient(object):
         """
         self.launch_context = self.auth.handle_callback(url, self.server)
         self._set_authorized(True)
-    
+        self.save_state()
+ 
     def reauthorize(self):
         """ Try to reauthorize with the server; handled by our `auth` instance.
         
@@ -179,7 +181,9 @@ class FHIRClient(object):
         self.app_id = state.get('app_id') or self.app_id
         self.launch_context = state.get('launch_context') or self.launch_context
         self.server = FHIRServer(state=state.get('server'))
-        self.auth = self._auth_for_type(state.get('auth_type'), state=state.get('auth'))
-        if self.auth is not None and self.auth.access_token is not None:
+        self.auth = FHIRAuth.create(state.get('auth_type'), app_id=self.app_id, state=state.get('auth'))
+        if self.auth is not None and self.auth.ready is not None:
             self.server.did_authorize(self.auth)
     
+    def save_state (self):
+        self._save_func(self.state)
